@@ -1,6 +1,6 @@
 package Dist::Zilla::Plugin::CheckPrereqsIndexed;
 # ABSTRACT: prevent a release if you have prereqs not found on CPAN
-$Dist::Zilla::Plugin::CheckPrereqsIndexed::VERSION = '0.018';
+$Dist::Zilla::Plugin::CheckPrereqsIndexed::VERSION = '0.019';
 use 5.10.0; # //
 use Moose;
 
@@ -55,23 +55,48 @@ sub before_release {
 
   require version;
 
-  my $prereqs_hash  = $self->zilla->prereqs->as_string_hash;
-
   my @skips = map {; qr/$_/ } @{ $self->skips };
 
   my $requirements = CPAN::Meta::Requirements->new;
 
-  # first level keys are phase; second level keys are types; we will just merge
-  # everything -- rjbs, 2011-08-18
-  for my $req_set (map { values %$_ } values %$prereqs_hash) {
-    REQ_PKG: for my $pkg (keys %$req_set) {
-      next if $NOT_INDEXED{ $pkg };
+  # find the package => version for all modules in this distribution
+  my $provides = $self->zilla->distmeta->{provides} // {};
+  my %self_modules = map { $_ => $provides->{$_}{version} } keys %$provides;
 
-      next if any { $pkg =~ $_ } @skips;
+  if (not keys %self_modules) {
+    (my $pkg = $self->zilla->name) =~ s/-/::/g;
+    $self->log_debug([ 'no "provides" metadata; guessing distribution contains module %s', $pkg ]);
+    %self_modules = ( $pkg => $self->zilla->version );
+  }
 
-      my $ver = $req_set->{$pkg};
+  for my $prereqs_hash (
+    $self->zilla->prereqs->as_string_hash,
+    (map { $_->{prereqs} } values %{ $self->zilla->distmeta->{optional_features} // {} }),
+  ) {
+    for my $phase (keys %$prereqs_hash) {
+      for my $type (keys %{$prereqs_hash->{$phase}}) {
+        REQ_PKG: for my $pkg (keys %{$prereqs_hash->{$phase}{$type}}) {
+          if ($NOT_INDEXED{ $pkg }) {
+            $self->log_debug([ 'skipping unindexed module %s', $pkg ]);
+            next;
+          }
 
-      $requirements->add_string_requirement($pkg => $ver);
+          if (any { $pkg =~ $_ } @skips) {
+            $self->log_debug([ 'explicitly skipping module %s', $pkg ]);
+            next;
+          }
+
+          my $ver = $prereqs_hash->{$phase}{$type}{$pkg};
+
+          # skip packages contained in the distribution we are releasing, from develop prereqs only
+          if ($phase eq 'develop' and exists $self_modules{$pkg} and $self_modules{$pkg} >= $ver) {
+            $self->log_debug([ 'skipping develop prereq on ourself (%s => %s)', $pkg, $ver ]);
+            next;
+          }
+
+          $requirements->add_string_requirement($pkg => $ver);
+        }
+      }
     }
   }
 
@@ -156,7 +181,7 @@ Dist::Zilla::Plugin::CheckPrereqsIndexed - prevent a release if you have prereqs
 
 =head1 VERSION
 
-version 0.018
+version 0.019
 
 =head1 OVERVIEW
 
